'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _colorLogger = require('color-logger');

var _colorLogger2 = _interopRequireDefault(_colorLogger);

var _Npm = require('./Npm');

var _Npm2 = _interopRequireDefault(_Npm);

var _Event = require('./Event');

var _Event2 = _interopRequireDefault(_Event);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

const fileSystem = _bluebird2.default.promisifyAll(_fs2.default);

/**
 * This class is usually the main part of the application.
 */

let PluginManager = function () {
  /**
   * Create a new instance.
   *
   * @param {Object}  [options]               options object
   * @param {Boolean} [options.debug]         debug flag
   * @param {String}  [options.context]       context name
   * @param {String}  [options.configFile]    set the config file path
   * @param {Boolean} [options.override=true] whether to override matching functions
   */
  function PluginManager() {
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, PluginManager);

    _colorLogger2.default.i('new PluginManager()', options);

    this._events = {};
    this._debug = options.debug || false;
    this._override = options.override || true;
    this._configFile = options.configFile || '.plugged-in.json';
    this._defaultMaxHandlers = null;
    this._context = options.context;

    _colorLogger2.default.debug = options.debug || false;
  }

  /**
   * Get the current application context.
   *
   * @returns {String} the context name
   */


  _createClass(PluginManager, [{
    key: 'init',


    /**
     * Initialize the application.
     *
     * Generates config file if it does not exist and loads plugins.
     *
     * @param {Object} plugin local event handlers
     * @param {Object} [npm]  optional Npm instance
     *
     * @returns {PluginManager} this plugin
     */
    value: (() => {
      var _ref = _asyncToGenerator(function* () {
        let plugin = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        let npm = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _Npm2.default({ debug: this._debug });

        _colorLogger2.default.i('PluginManager.init()');

        let data;

        try {
          const exists = fileSystem.existsSync(this._configFile);

          if (exists === true) {
            data = yield fileSystem.readFileAsync(this._configFile);

            data = JSON.parse(data);

            yield this.addPlugins(data.plugins);
          } else {
            _colorLogger2.default.d('.plugged-in.json does not exists');

            data = yield npm.generateConfig(this);
          }

          if (typeof this._context === 'undefined') {
            this._context = data.context;
          }
        } catch (error) {
          _colorLogger2.default.e('PluginManager.init()', error.message);
        }

        // Add local plugin last so that it can override externals if necessary
        yield this.addPlugins([plugin]);

        this.dispatch('postInit', this);

        return this;
      });

      function init(_x2, _x3) {
        return _ref.apply(this, arguments);
      }

      return init;
    })()

    /**
     * Helper method to dispatch events.
     *
     * @param {String} eventName the event name
     * @param {Object} [context] optional event context object
     *
     * @returns {Object} the modified data object
     */

  }, {
    key: 'dispatch',
    value: (() => {
      var _ref2 = _asyncToGenerator(function* (eventName, context) {
        if (typeof eventName === 'undefined') {
          throw new Error('Event name not provided');
        }

        if (typeof context === 'undefined') {
          context = {};
        }

        const event = new _Event2.default({ name: eventName, context: context });

        _colorLogger2.default.i(`Dispatching ${ event.name }...`);

        const handlers = this.handlers(event.name);

        yield _bluebird2.default.mapSeries(handlers, (() => {
          var _ref3 = _asyncToGenerator(function* (func) {
            console.log('Executing', func.name);

            try {
              yield func(event);
            } catch (error) {
              throw error;
            }
          });

          return function (_x8) {
            return _ref3.apply(this, arguments);
          };
        })());

        return context;
      });

      function dispatch(_x6, _x7) {
        return _ref2.apply(this, arguments);
      }

      return dispatch;
    })()

    /**
     * Get list of handlers for an event.
     *
     * @param {String} eventName the event name
     *
     * @returns {Function[]} list of handlers
     */

  }, {
    key: 'handlers',
    value: function handlers(eventName) {
      const result = this._events[eventName];

      if (typeof result === 'undefined') {
        return [];
      }

      return result;
    }

    /**
     * Adds a new handler to the event list.
     *
     * @param {String}   eventName the event name
     * @param {Function} handler   the callback function
     *
     * @returns {Object} this instance
     */

  }, {
    key: 'addHandler',
    value: function addHandler(eventName, handler) {
      _colorLogger2.default.i('addHandler', eventName, handler);

      const handlers = this.handlers(eventName);

      if (handlers.length === 0) {
        this._events[eventName] = [];
      }

      if (this._defaultMaxHandlers !== null && handlers.length + 1 > this._defaultMaxHandlers) {
        throw new Error(`Exceeded maximum number of handlers for '${ eventName }' event`);
      }

      this._events[eventName].push(handler);

      return this;
    }

    /**
     * Get max event handlers.
     *
     * @returns {Integer} max number of handlers
     */

  }, {
    key: 'getMaxHandlers',
    value: function getMaxHandlers() {
      return this._defaultMaxHandlers;
    }

    /**
     * Set max number of event handlers.
     *
     * @param {Integer} count max number
     *
     * @returns {Object} this instance
     */

  }, {
    key: 'setMaxHandlers',
    value: function setMaxHandlers(count) {
      if (Number.isNaN(count)) {
        throw new Error(`Count: '${ count }' is not a number`);
      }

      this._defaultMaxHandlers = count;

      return this;
    }

    /**
     * Determines if a handler exists for an event.
     *
     * @param {String}          event   the event name
     * @param {Function|String} handler the function or its name
     *
     * @returns {Boolean} true if exists, otherwise false
     */

  }, {
    key: 'hasHandler',
    value: function hasHandler(event, handler) {
      const handlers = this.handlers(event).map(handle => handle.name);

      if (typeof handler === 'function') {
        return handlers.indexOf(handler.name) !== -1;
      }

      return handlers.indexOf(handler) !== -1;
    }

    /**
     * Determines if an event has any handlers.
     *
     * @param {String} event the event name
     *
     * @returns {Boolean} true or false
     */

  }, {
    key: 'hasHandlers',
    value: function hasHandlers(event) {
      if (typeof event === 'undefined') {
        throw new Error('Undefined event name');
      }

      return this.handlers(event).length > 0;
    }

    /**
     * Removes a handler for an event that matches the handler function name.
     *
     * @param {String}          eventName the event name
     * @param {Function|String} handler   the function or its name
     *
     * @returns {PluginManager} this instance
     */

  }, {
    key: 'removeHandler',
    value: function removeHandler(eventName, handler) {
      let name = handler;

      if (typeof name === 'function') {
        name = handler.name;
      }

      let index = 0;

      this.handlers(eventName).forEach(handle => {
        if (handle.name === name) {
          this._events[eventName].splice([index], 1);
        }

        index += 1;
      });

      return this;
    }

    /**
     * Deletes all handlers for an event.
     *
     * @param {String} eventName the event name to removes
     *
     * @returns {Object} this instance
     */

  }, {
    key: 'removeAllHandlers',
    value: function removeAllHandlers(eventName) {
      this._events[eventName] = [];

      return this;
    }

    /**
     * Add local plugins.
     *
     * @param {Object[]} plugins          list of plugins
     * @param {Object}   plugins.provides the provides object
     *
     * @returns {PluginManager} this instance
     */

  }, {
    key: 'addPlugins',
    value: (() => {
      var _ref4 = _asyncToGenerator(function* (plugins) {
        var _this = this;

        if (Array.isArray(plugins) === false) {
          throw new Error('Plugins must be an array');
        }

        yield _bluebird2.default.mapSeries(plugins, (() => {
          var _ref5 = _asyncToGenerator(function* (plugin) {
            return _this._processPlugin(plugin);
          });

          return function (_x10) {
            return _ref5.apply(this, arguments);
          };
        })());

        return this;
      });

      function addPlugins(_x9) {
        return _ref4.apply(this, arguments);
      }

      return addPlugins;
    })()

    /**
     * Add plugin handlers.
     *
     * @param {Object} plugin          the plugin
     * @param {Object} plugin.provides the provides object
     *
     * @returns {PluginManager} this instance
     *
     * @private
     */

  }, {
    key: '_processPlugin',
    value: (() => {
      var _ref6 = _asyncToGenerator(function* (plugin) {
        var _this2 = this;

        try {
          if (typeof plugin.provides === 'undefined') {
            return this;
          }

          const pluginEvents = Object.keys(plugin.provides);

          yield _bluebird2.default.mapSeries(pluginEvents, (() => {
            var _ref7 = _asyncToGenerator(function* (eventName) {
              return _this2._processEvent(plugin, eventName);
            });

            return function (_x12) {
              return _ref7.apply(this, arguments);
            };
          })());
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }

        return this;
      });

      function _processPlugin(_x11) {
        return _ref6.apply(this, arguments);
      }

      return _processPlugin;
    })()

    /**
     * Add plugin event.
     *
     * @param {Object} plugin    the plugin
     * @param {String} eventName the event name
     *
     * @returns {PluginManager} this instance
     *
     * @private
     */

  }, {
    key: '_processEvent',
    value: (() => {
      var _ref8 = _asyncToGenerator(function* (plugin, eventName) {
        var _this3 = this;

        try {
          let funcs = plugin.provides[eventName];

          if (Array.isArray(funcs) !== true) {
            funcs = [funcs];
          }

          yield _bluebird2.default.mapSeries(funcs, (() => {
            var _ref9 = _asyncToGenerator(function* (func) {
              return _this3._processHandler(plugin, eventName, func);
            });

            return function (_x15) {
              return _ref9.apply(this, arguments);
            };
          })());
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }

        return this;
      });

      function _processEvent(_x13, _x14) {
        return _ref8.apply(this, arguments);
      }

      return _processEvent;
    })()

    /**
     * Add plugin handler.
     *
     * @param {Object}   plugin    the plugin
     * @param {String}   eventName the event name
     * @param {Function} handler   the handler
     *
     * @returns {PluginManager} this instance
     *
     * @private
     */

  }, {
    key: '_processHandler',
    value: (() => {
      var _ref10 = _asyncToGenerator(function* (plugin, eventName, handler) {
        try {
          let func = handler;

          if (typeof handler === 'string') {
            func = yield this._getCallback(plugin.name, handler);
          }

          if (func === null) {
            return this;
          }

          this._consolidateHandlers(eventName, func).addHandler(eventName, func);
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }

        return this;
      });

      function _processHandler(_x16, _x17, _x18) {
        return _ref10.apply(this, arguments);
      }

      return _processHandler;
    })()

    /**
     * This removes existing matching handler for an event if necessary.
     *
     * @param {String}   eventName event name
     * @param {Function} handler   the handler
     *
     * @returns {PluginManager} this instance
     *
     * @private
     */

  }, {
    key: '_consolidateHandlers',
    value: function _consolidateHandlers(eventName, handler) {
      if (this.hasHandler(eventName, handler) === true && this._override === true) {
        this.removeHandler(eventName, handler);
      }

      return this;
    }

    /**
     * Get callback for plugin service.
     *
     * @param {Object}   pluginName the plugin
     * @param {String}   funcName   the name of the event
     *
     * @returns {Function} the handler
     *
     * @private
     */

  }, {
    key: '_getCallback',
    value: (() => {
      var _ref11 = _asyncToGenerator(function* (pluginName, funcName) {
        try {
          const pg = require(`${ pluginName }`); // eslint-disable-line global-require

          return pg[funcName];
        } catch (error) {
          _colorLogger2.default.e(`Unsupported action '${ funcName }'`, error);
        }

        return null;
      });

      function _getCallback(_x19, _x20) {
        return _ref11.apply(this, arguments);
      }

      return _getCallback;
    })()
  }, {
    key: 'context',
    get: function get() {
      return this._context;
    }

    /**
     * Determine if running in debug mode.
     *
     * @returns {Boolean} true or false
     */

  }, {
    key: 'debug',
    get: function get() {
      return this._debug;
    }

    /**
     * Get logger.
     *
     * @returns {Object} the logger instance
     */

  }, {
    key: 'log',
    get: function get() {
      return _colorLogger2.default;
    }
  }]);

  return PluginManager;
}();

exports.default = PluginManager;